@@ -5,11 +5,10 @@ use strict;
 use warnings;
 use utf8;
 
-our $VERSION = 0.12;
+our $VERSION = 0.13;
 
 use Encode::Detect::CJK qw/detect/;
 use Encode;
-use Parallel::ForkManager;
 use WWW::Mechanize;
 
 sub new {
@@ -45,20 +44,11 @@ sub _init_browser {
     return $http;
 } ## end sub init_browser
 
+use threads;
+use threads::shared;
+use Thread::Semaphore;
 sub request_urls {
     my ($self, $arr, %opt) = @_;
-
-    my @res;
-    my $pm = Parallel::ForkManager->new( $self->{max_process_num} );
-    $pm->run_on_finish(
-        sub {
-            my ( $pid, $exit_code, $ident, $exit, $core, $data ) = @_;
-            return unless($data);
-            my ($i, $r) = @$data;
-            $res[$i] = $r;
-        }
-    );
-
     $opt{request_sub} ||= sub {
         my ($x) = @_;
         my ($url, $post_var) = exists $opt{url_sub} ? $opt{url_sub}->($x) : 
@@ -68,19 +58,93 @@ sub request_urls {
         return $self->request_url($url, $post_var);
     };
 
-    for my $i (0 .. $#$arr){
-        my $pid = $pm->start and next;
+    my $sem = Thread::Semaphore->new($self->{max_process_num});
+    my $work_func = sub {
+        my ($i) = @_;
         my $x = $arr->[$i];
-
         my $s = $opt{request_sub}->($x);
         $s = $opt{deal_sub}->($x, $s) if(exists $opt{deal_sub});
+        $sem->up;
+        return [$i, $s ];
+    };
+    my @threads = map {
+        $sem->down;
+        threads->create($work_func, $_);
+    } 0..$#$arr;
 
-        $pm->finish( 0, [ $i, $s ] );
+    my @res;
+    for (@threads){
+        my $r = $_->join;
+        $res[$r->[0]] = $r->[1];
     }
-    $pm->wait_all_children;
     return \@res;
 }
 
+#use Coro;
+#sub request_urls {
+    #my ($self, $arr, %opt) = @_;
+    #$opt{request_sub} ||= sub {
+        #my ($x) = @_;
+        #my ($url, $post_var) = exists $opt{url_sub} ? $opt{url_sub}->($x) : 
+        #ref($x) eq 'HASH' ? ( $x->{url}, $x->{post_var} )  :
+        #ref($x) eq 'ARRAY' ? @$x : 
+        #($x, undef);
+        #return $self->request_url($url, $post_var);
+    #};
+
+    #my @tasks;
+    #my @res;
+    #for my $i (0 .. $#$arr){
+        #push @tasks, async {
+            #my $x = $arr->[$i];
+            #my $s = $opt{request_sub}->($x);
+            #$s = $opt{deal_sub}->($x, $s) if(exists $opt{deal_sub});
+            #$res[$i] = $s;
+        #}
+    #}
+    #for (@tasks){
+        #$_->join;
+    #}
+    #return \@res;
+#}
+
+#use Parallel::ForkManager;
+#sub request_urls {
+    #my ($self, $arr, %opt) = @_;
+
+    #my @res;
+    #my $pm = Parallel::ForkManager->new( $self->{max_process_num} );
+    #$pm->run_on_finish(
+        #sub {
+            #my ( $pid, $exit_code, $ident, $exit, $core, $data ) = @_;
+            #return unless($data);
+            #my ($i, $r) = @$data;
+            #$res[$i] = $r;
+        #}
+    #);
+
+    #$opt{request_sub} ||= sub {
+        #my ($x) = @_;
+        #my ($url, $post_var) = exists $opt{url_sub} ? $opt{url_sub}->($x) : 
+        #ref($x) eq 'HASH' ? ( $x->{url}, $x->{post_var} )  :
+        #ref($x) eq 'ARRAY' ? @$x : 
+        #($x, undef);
+        #return $self->request_url($url, $post_var);
+    #};
+
+    #for my $i (0 .. $#$arr){
+        #my $pid = $pm->start and next;
+        #my $x = $arr->[$i];
+
+        #my $s = $opt{request_sub}->($x);
+        #$s = $opt{deal_sub}->($x, $s) if(exists $opt{deal_sub});
+
+        #$pm->finish( 0, [ $i, $s ] );
+    #}
+    #$pm->wait_all_children;
+    #return \@res;
+#}
+
 sub request_url {
     my ( $self, $url, $post_data ) = @_;
 
